using System;
using System.Globalization;
using com.paypal.sdk.services;
using com.paypal.soap.api;
using com.paypal.sdk.profiles;

namespace SpiceLogic.PayPalProSOAP.Core
{
    public class DoDirectPayment
    {
        /// <summary>
        /// Pays the now.
        /// </summary>
        /// <param name="currencyCode">The currency code.</param>
        /// <param name="paymentAmount">The payment amount.</param>
        /// <param name="buyerLastName">Last name of the buyer.</param>
        /// <param name="buyerFirstName">First name of the buyer.</param>
        /// <param name="buyerAddress1">The buyer address1.</param>
        /// <param name="buyerAddress2">The buyer address2.</param>
        /// <param name="buyerCity">The buyer city.</param>
        /// <param name="buyerState">State of the buyer.</param>
        /// <param name="buyerZipCode">The buyer zip code.</param>
        /// <param name="buyerCountry">The buyer country. (3 character country code)</param>
        /// <param name="countryCode">The country code.</param>
        /// <param name="creditCardType">Type of the credit card.</param>
        /// <param name="creditCardNumber">The credit card number.</param>
        /// <param name="CVV2">The 3 digit CVV2 code from the back side of the card.</param>
        /// <param name="expMonth">The exp month.</param>
        /// <param name="expYear">The exp year.</param>
        /// <param name="paymentAction">The payment action.</param>
        /// <param name="buyerIPAddress">The buyer IP address.</param>
        /// <returns></returns>
        public DoDirectPaymentResponseType PayNow(CurrencyCodeType currencyCode, decimal paymentAmount, 
            string buyerLastName, 
            string buyerFirstName, 
            string buyerAddress1, 
            string buyerAddress2, 
            string buyerCity, 
            string buyerState, 
            string buyerZipCode, 
            string buyerCountry,
            CountryCodeType countryCode,
            string creditCardType, 
            string creditCardNumber, 
            string CVV2, 
            int expMonth, 
            int expYear, 
            PaymentActionCodeType paymentAction,
            string buyerIPAddress)
        {
            try
            {
                CallerServices caller = new CallerServices();

                IAPIProfile profile = ProfileFactory.createSignatureAPIProfile();
                /*
                 WARNING: Do not embed plaintext credentials in your application code.
                 Doing so is insecure and against best practices.
                 Your API credentials must be handled securely. Please consider
                 encrypting them for use in any production environment, and ensure
                 that only authorized individuals may view or modify them.
                 */

                // Set up your API credentials, PayPal end point, and API version.
                profile.APIUsername = ConfigData._apiUserName;
                profile.APIPassword = ConfigData._apiPassword;
                profile.APISignature = ConfigData._apiSignature;
                profile.Environment = ConfigData.PayPalEnvironment;
                caller.APIProfile = profile;

                // Create the request object.
                DoDirectPaymentRequestType pp_Request = new DoDirectPaymentRequestType
                {
                    Version = ConfigData.APIVersion,
                    DoDirectPaymentRequestDetails = new DoDirectPaymentRequestDetailsType
                    {
                        IPAddress = buyerIPAddress,
                        MerchantSessionId = ConfigData.MerchantSessionId,
                        PaymentAction = paymentAction,
                        CreditCard = new CreditCardDetailsType { CreditCardNumber = creditCardNumber }
                    }
                };

                // Add request-specific fields to the request.
                // Create the request details object.

                switch (creditCardType)
                {
                    case "Visa":
                        pp_Request.DoDirectPaymentRequestDetails.CreditCard.CreditCardType = CreditCardTypeType.Visa;
                        break;
                    case "MasterCard":
                        pp_Request.DoDirectPaymentRequestDetails.CreditCard.CreditCardType = CreditCardTypeType.MasterCard;
                        break;
                    case "Discover":
                        pp_Request.DoDirectPaymentRequestDetails.CreditCard.CreditCardType = CreditCardTypeType.Discover;
                        break;
                    case "Amex":
                        pp_Request.DoDirectPaymentRequestDetails.CreditCard.CreditCardType = CreditCardTypeType.Amex;
                        break;
                }

                pp_Request.DoDirectPaymentRequestDetails.CreditCard.CVV2 = CVV2;
                pp_Request.DoDirectPaymentRequestDetails.CreditCard.ExpMonth = expMonth;
                pp_Request.DoDirectPaymentRequestDetails.CreditCard.ExpYear = expYear;
                pp_Request.DoDirectPaymentRequestDetails.CreditCard.ExpMonthSpecified = true;
                pp_Request.DoDirectPaymentRequestDetails.CreditCard.ExpYearSpecified = true;
                pp_Request.DoDirectPaymentRequestDetails.CreditCard.CardOwner = new PayerInfoType
                {
                    Payer = "",
                    PayerID = "",
                    PayerStatus = PayPalUserStatusCodeType.verified,
                    PayerCountry = countryCode,
                    Address = new AddressType
                    {
                        Street1 = buyerAddress1,
                        Street2 = buyerAddress2,
                        CityName = buyerCity,
                        StateOrProvince = buyerState,
                        PostalCode = buyerZipCode,
                        CountryName = buyerCountry,
                        Country = countryCode,
                        CountrySpecified = true
                    },

                    PayerName = new PersonNameType { FirstName = buyerFirstName, LastName = buyerLastName }
                };

                pp_Request.DoDirectPaymentRequestDetails.PaymentDetails = new PaymentDetailsType
                {
                    OrderTotal = new BasicAmountType
                    {
                        currencyID = currencyCode,
                        Value = paymentAmount.ToString(CultureInfo.InvariantCulture)
                    }
                };

                // Execute the API operation and obtain the response.
                DoDirectPaymentResponseType pp_response = (DoDirectPaymentResponseType)caller.Call("DoDirectPayment", pp_Request);
                return pp_response;
            }
            catch(Exception)
            {
                DoDirectPaymentResponseType myReturn = new DoDirectPaymentResponseType {Ack = AckCodeType.Failure};               
                return myReturn;
            }
        }
    }
}
