﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;
using com.paypal.soap.api;

namespace WebApplication1
{
    public partial class Default : Page
    {
        private const string paymentSuccessUrl = "http://www.google.com"; //todo : Change this URL according to your need.
        private const string paymentFailedUrl = "http://www.microsoft.com"; //todo: Change this URL according to your need.

        /// <summary>
        /// Handles the Load event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {                
                btnPayNow.Style.Add("cursor", "pointer");
                btnContinue1.Style.Add("cursor", "pointer");
                btnContinue2.Style.Add("cursor", "pointer");
                cmbStates.Items.Add(new ListItem { Text = "USA", Value=""});
                cmbStates.Items.Add(new ListItem { Text = "------------", Value=""});
                Dictionary<string, string> usStates = GetProvinces("US");
                foreach (KeyValuePair<string, string> aState in usStates)
                    cmbStates.Items.Add(new ListItem(aState.Value, aState.Key));
                cmbStates.Items.Add(new ListItem { Text = "------------", Value = "" });
                cmbStates.Items.Add(new ListItem { Text = "Canada", Value = "" });
                cmbStates.Items.Add(new ListItem { Text = "------------", Value = "" });
                Dictionary<string, string> caStates = GetProvinces("CA");
                foreach (KeyValuePair<string, string> aState in caStates)
                    cmbStates.Items.Add(new ListItem(aState.Value, aState.Key));
            }
        }

        /// <summary>
        /// Handles the Click event of the btnPayNow control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void btnPayNow_Click(object sender, EventArgs e)
        {
            SpiceLogic.PayPalProSOAP.Core.DoDirectPayment myPaymentMaker =
                new SpiceLogic.PayPalProSOAP.Core.DoDirectPayment();

            const string myIP = "127.0.0.1";
            //todo: You should change it to -> HttpContext.Current.Request.UserHostAddress for your Production Server ;

            string state = cmbStates.SelectedValue;
            string currencyCode = txtCurrency.Text;
            string amountToPay = txtAmountToPay.Text;
            DoDirectPaymentResponseType pp_response =
                myPaymentMaker.PayNow((CurrencyCodeType) Enum.Parse(typeof (CurrencyCodeType), currencyCode),
                    decimal.Parse(amountToPay),
                    txtLastName.Text,
                    txtFirstName.Text, txtStAddr1.Text, txtStAddr2.Text, txtCity.Text,
                    state,
                    txtZipCode.Text,
                    cmbCountries.Text,
                    (CountryCodeType) Enum.Parse(typeof (CountryCodeType), cmbCountries.SelectedValue),
                    cmbCreditCardType.SelectedValue,
                    txtCreditCardNumber.Text, txtCvv2Code.Text,
                    int.Parse(cmbExpiryDateMonth.SelectedValue),
                    int.Parse(cmbExpiryDateYear.SelectedValue),
                    PaymentActionCodeType.Sale,
                    myIP);

            string successUrl = "";

            if (pp_response.Ack == AckCodeType.Success)
            {
                List<string> myResult = new List<string>
                {
                    "Status of this Transaction : " + pp_response.Ack,
                    "Transaction ID : " + pp_response.TransactionID,
                    string.Format("Paid Amount : {0} {1}", pp_response.Amount.currencyID,
                        pp_response.Amount.Value)
                };

                string paymentStatus = String.Join("<br />", myResult);
                lblStatus.Text = paymentStatus;

                btnPayNow.Visible = false;
                btnContinue1.Visible = true;
                btnContinue2.Visible = false;

                successUrl = String.Format("{0}?txn_id={1}&amount={2}&currency={3}", paymentSuccessUrl,
                    pp_response.TransactionID, pp_response.Amount.Value, pp_response.Amount.currencyID);
            }
            else
            {
                StringBuilder allErrorsBuilder = new StringBuilder();
                if (pp_response.Errors != null)
                    foreach (ErrorType anError in pp_response.Errors)
                        allErrorsBuilder.Append("<br /> - " + anError.LongMessage);

                lblStatus.Text = "FAILED";
                lblStatus.Text += "<br /><br />" + allErrorsBuilder;

                btnPayNow.Text = "Retry";
                btnContinue2.Visible = true;
            }

            btnContinue1.Attributes["onClick"] = string.Format("window.location='{0}'", successUrl);
            btnContinue2.Attributes["onClick"] = string.Format("window.location='{0}'", paymentFailedUrl);
        }

        /// <summary>
        /// Gets the provinces for the country.
        /// </summary>
        /// <param name="country_code">The country_code.</param>
        /// <returns></returns>
        private static Dictionary<string, string> GetProvinces(string country_code)
        {
            string state_data_file = HttpContext.Current.Server.MapPath(string.Format("~/App_Data/States/{0}.xml", country_code));
            if (!File.Exists(state_data_file))
                return null;

            XmlDocument myDocument = new XmlDocument();
            myDocument.Load(state_data_file);
            XmlNodeList provinceNodes = myDocument.SelectNodes("/States/state");
            Dictionary<string, string> myReturnValue = new Dictionary<string, string>();

            if (provinceNodes != null)
            {
                foreach (XmlNode aNode in provinceNodes)
                {
                    if (aNode.Attributes != null)
                        myReturnValue.Add(aNode.Attributes["code"].Value, aNode.Attributes["name"].Value);
                }
            }

            return myReturnValue;
        }
    }
}